@@ -267,7 +267,7 @@ gb_bonobo_ui_toolbar_build (GbBonoboUIToolbar *buit, GbFolder *f)
 	buiti = g_hash_table_lookup (p->bookmark_to_main_buiti, f);
 	if (!buiti) 
 	{
-		gchar *tbname = g_strdup_printf ("BookmarkToolbar%20s%ld", 
+		gchar *tbname = g_strdup_printf ("BookmarkToolbar%20s%s", 
 						 GB_BOOKMARK (f)->name, 
 						 GB_BOOKMARK (f)->id);
 		buiti = buit_item_new (buit, GB_BOOKMARK (f), TRUE);
