@@ -30,6 +30,7 @@
 #include "gul-gui.h"
 #include "bookmarks-widgets-private.h"
 #include "bookmarks-gtk-menu.h"
+#include "bookmarks-dnd.h"
 
 #include <gtk/gtktogglebutton.h>
 
@@ -151,6 +152,7 @@ gb_folder_tb_widget_rebuild_impl (GbTbWidget *w)
 	GTK_WIDGET_SET_FLAGS (GTK_BUTTON (p->mainwid), GTK_CAN_FOCUS);
 
 	gb_tb_widget_setup_context_menu (w, p->mainwid);
+	gb_bookmark_dnd_drag_dest_set (p->mainwid, GB_BOOKMARK (folder));
 }
 
 static gboolean
