@@ -25,18 +25,20 @@
 #include "bookmarks-toolbar-widgets.h"
 #include "bookmarks-default-icon-provider.h"
 #include "bookmarks-util.h"
+#include "bookmarks-iterator.h"
 #include "galeon-marshal.h"
 #include "pixbuf-cache.h"
 #include "gul-general.h"
 #include "gul-string.h"
 #include "gul-gobject-misc.h"
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <string.h>
 #include "galeon-autocompletion-source.h"
 
 #define NOT_IMPLEMENTED g_warning ("not implemented: " G_STRLOC);
 
 //#define DEBUG_MSG(x) g_print x
-#define DEBUG_MSG(x)
+#define DEBUG_MSG(x) 
 
 /**
  * Private functions, only availble from this file
@@ -55,6 +57,7 @@ static void		gb_bookmark_set_autocompletion_source_set_basic_key
 							(GaleonAutocompletionSource *source,
 							 const gchar *basic_key);
 static void		gb_bookmark_set_emit_autocompletion_source_data_changed (GbBookmarkSet *gh);
+static void		gb_bookmark_set_tree_changed	(GbBookmarkSet *set);
 static void		gb_site_class_init		(GbSiteClass *klass);
 static void		gb_site_init			(GbSite *b);
 static void		gb_folder_class_init		(GbFolderClass *klass);
@@ -121,6 +124,7 @@ static GbTbWidget *	gb_folder_create_toolbar_widget_impl (GbBookmark *b);
 static GbTbWidget *	gb_site_create_toolbar_widget_impl (GbBookmark *b);
 static GbTbWidget *	gb_separator_create_toolbar_widget_impl (GbBookmark *b);
 static GbTbWidget *	gb_smart_site_create_toolbar_widget_impl (GbBookmark *b);
+static GbTbWidget *	gb_alias_placeholder_create_toolbar_widget_impl (GbBookmark *b);
 static void		gb_bookmark_init_icons		(void);
 static gboolean		gb_bookmark_set_set_auto_save_cb (gpointer data);
 #define gb_bookmark_set_needs_saving(b) if (b && ((GbBookmark *) b)->set) \
@@ -132,6 +136,8 @@ static char *		gb_bookmark_set_fix_galeon1_mess_string		(const gchar *s);
 static gchar *		gb_smart_site_get_smarturl_only		(GbSmartSite *b);
 static gchar *		gb_smart_site_get_options		(GbSmartSite *b);
 static void		gb_smart_site_set_options		(GbSmartSite *b, gchar *options);
+static GSList *		gb_folder_list_children_reversed	(GbFolder *f);
+static GbBookmark *	gb_folder_get_child_or_alias		(GbFolder *f, GbBookmark *c);
 
 #define BOOKMARKS_NUM_BACKUPS 5
 
@@ -280,14 +286,18 @@ gb_bookmark_finalize_impl (GObject *o)
 	
 	g_assert (b->parent == NULL);
 
-	if (b->set && b->id)
-	{
-		g_hash_table_remove (b->set->id_to_bookmark, GINT_TO_POINTER (b->id));
-		gb_bookmark_set_emit_autocompletion_source_data_changed (b->set);
-	}
-	
 	if (!gb_bookmark_is_alias (b))
 	{
+		if (b->set && b->id)
+		{
+			g_hash_table_remove (b->set->id_to_bookmark, b->id);
+		}
+
+		if (b->set)
+		{
+			gb_bookmark_set_emit_autocompletion_source_data_changed (b->set);
+		}
+
 #ifdef NICK_HASHTABLE
 		if (b->set)
 		{
@@ -307,6 +317,7 @@ gb_bookmark_finalize_impl (GObject *o)
 		}
 		else
 		{
+			g_free (b->id);
 			g_free (b->name);
 			g_free (b->nick);
 			g_free (b->pixmap_file);
@@ -354,7 +365,8 @@ gb_bookmark_copy_impl_do (GbBookmark *b, GbBookmark *c)
 	c->next = NULL;
 	c->prev = NULL;
 	c->alias_of = NULL;
-	c->id = 0;
+	c->alias = NULL;
+	c->id = NULL;
 	g_free (c->name);
 	c->name = g_strdup (b->name);
 	g_free (c->nick);
@@ -396,14 +408,16 @@ gb_bookmark_alias_create_impl_do (GbBookmark *b, GbAliasPlaceholder *ap,
 	GbBookmark *alias_data = (GbBookmark *) ap;
 	
 	/* make sure the bookmark has an id */
-	if (b->id == 0)
+	if (b->id == NULL)
 	{
 		static gulong last_generated_id = 0;
+		char id[32];
 		do {
-			last_generated_id++;
+			/* ID's have to begin with a Letter to be valid XML */
+			snprintf (id, sizeof(id), "id%lu", last_generated_id++);
 			if (!b->set 
 			    || g_hash_table_lookup (b->set->id_to_bookmark, 
-						    GINT_TO_POINTER (last_generated_id)) == NULL)
+						    id) == NULL)
 			{
 				break;
 			}
@@ -412,8 +426,7 @@ gb_bookmark_alias_create_impl_do (GbBookmark *b, GbAliasPlaceholder *ap,
 		{
 			g_warning ("Could not find an unique id. You have a lot of bookmarks!");
 		}
-		gb_bookmark_set_id (b, last_generated_id);
-		
+		gb_bookmark_set_id (b, id);
 	}
 
 	if (alias->set)
@@ -422,6 +435,7 @@ gb_bookmark_alias_create_impl_do (GbBookmark *b, GbAliasPlaceholder *ap,
 	}
 
 	alias->set = b->set;
+	alias->id = b->id;
 
 	alias->alias_of = b;
 	alias->alias = b->alias;
@@ -462,11 +476,13 @@ gb_bookmark_init (GbBookmark *b)
 {
 	b->parent = NULL;
 	b->next = NULL;
+	b->id = NULL;
 	b->name = g_strdup ("");
 	b->nick = g_strdup ("");
 	b->notes = g_strdup ("");
 	b->pixmap_file = g_strdup ("");
 	b->set = NULL;
+	b->xbel_node = NULL;
 	
 #ifdef DEBUG_REF
 	g_print ("live_bookmarks: %d+\n", ++live_bookmarks);
@@ -489,9 +505,9 @@ gb_bookmark_set_set_impl (GbBookmark *b, GbBookmarkSet *set)
 	
 	oldset = b->set;
 
-	if (oldset && b->id)
+	if (oldset && b->id && !gb_bookmark_is_alias (b))
 	{
-		g_hash_table_remove (oldset->id_to_bookmark, GINT_TO_POINTER (b->id));
+		g_hash_table_remove (oldset->id_to_bookmark, b->id);
 		gb_bookmark_set_emit_autocompletion_source_data_changed (oldset);
 	}
 
@@ -502,13 +518,15 @@ gb_bookmark_set_set_impl (GbBookmark *b, GbBookmarkSet *set)
 	}
 #endif
 
+	gb_bookmark_set_xbel_node (b, NULL);
+
 	b->set = set;
 
 	/* add to hashtables... */
-	if (b->set && b->id)
+	if (b->set && b->id && !gb_bookmark_is_alias (b))
 	{
 		/* FIXME: i should check that the id is unique... */
-		g_hash_table_insert (set->id_to_bookmark, GINT_TO_POINTER (b->id), b);
+		g_hash_table_insert (set->id_to_bookmark, b->id, b);
 		gb_bookmark_set_emit_autocompletion_source_data_changed (b->set);
 	}
 
@@ -521,33 +539,51 @@ gb_bookmark_set_set_impl (GbBookmark *b, GbBookmarkSet *set)
 }
 
 void
-gb_bookmark_set_id (GbBookmark *b, gulong val)
+gb_bookmark_set_id (GbBookmark *b, const gchar *val)
 {
+	gchar *newval;
 	g_return_if_fail (GB_IS_BOOKMARK (b));
 	
-	if (GB_IS_BOOKMARK_SET (b->set)) 
-	{
-		if (b->id != 0)
-		{
-			g_hash_table_remove (b->set->id_to_bookmark, GINT_TO_POINTER (b->id));
-		}
-	}
-	
-	b->id = val;
+	b = gb_bookmark_real_bookmark (b);
 
 	if (GB_IS_BOOKMARK_SET (b->set)) 
 	{
-		if (val)
+		if (b->id != NULL)
 		{
-			g_hash_table_insert (b->set->id_to_bookmark, GINT_TO_POINTER (b->id), b);
+			g_hash_table_remove (b->set->id_to_bookmark, b->id);
+		}
+	}
+	
+	g_free (b->id);
+	newval = val ? g_strdup (val) : NULL;
+	b->id = newval;
+
+	if (GB_IS_BOOKMARK_SET (b->set)) 
+	{
+		if (b->id != NULL)
+		{
+			g_hash_table_insert (b->set->id_to_bookmark, b->id, b);
 		}
 	}
 
 	gb_bookmark_set_needs_saving (b);
 
-	/* TODO: update alias */
+	do {
+		b->id = newval;
+		g_signal_emit (b, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
+		gb_folder_emit_child_modified (b->parent, b);
+	} while ((b = b->alias) != NULL);
+}
 
-	g_signal_emit (b, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
+void
+gb_bookmark_set_xbel_node (GbBookmark *b, xmlNodePtr node)
+{
+	g_return_if_fail (GB_IS_BOOKMARK (b));
+	g_return_if_fail (GB_IS_BOOKMARK_SET (b->set) || node == NULL);
+	g_return_if_fail (node == NULL 
+			  || node->doc == b->set->xbel_doc);
+
+	b->xbel_node = node;
 }
 
 void
@@ -575,7 +611,6 @@ gb_bookmark_set_name (GbBookmark *b, const gchar *val)
 		g_signal_emit (b, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
 		gb_folder_emit_child_modified (b->parent, b);
 	} while ((b = b->alias) != NULL);
-
 }
 
 void
@@ -750,6 +785,115 @@ gb_bookmark_unparent (GbBookmark *b)
 	gb_bookmark_unparent_internal (b, TRUE);
 }
 
+static GbBookmark *
+gb_bookmark_get_parented_alias (GbBookmark *b)
+{
+	GbBookmark *alias;
+	g_return_val_if_fail (!gb_bookmark_is_alias (b), NULL);
+
+	for (alias = b->alias; alias; alias = alias->alias)
+	{
+		if (alias->parent)
+		{
+			return alias;
+		}
+	}
+	return NULL;
+}
+
+static GbBookmark *
+gb_bookmark_get_parented_alias_not_under (GbBookmark *b, GbFolder *f)
+{
+	GbBookmark *alias;
+	g_return_val_if_fail (!gb_bookmark_is_alias (b), NULL);
+
+	for (alias = b->alias; alias; alias = alias->alias)
+	{
+		if (alias->parent)
+		{
+			if (!gb_folder_is_ancestor (f, alias))
+			{
+				return alias;
+			}
+		}
+	}
+	return NULL;
+}
+
+static void
+gb_folder_save_descendants (GbFolder *f, GbFolder *unsafe)
+{
+	GbBookmark *child;
+	g_return_if_fail (!gb_bookmark_is_alias (f));
+
+	DEBUG_MSG (("Trying to save descendants of %s not under %s\n", GB_BOOKMARK (f)->name, GB_BOOKMARK (unsafe)->name));
+	
+	for (child = f->child; child; child = child->next)
+	{
+		if (!gb_bookmark_is_alias (child) && gb_bookmark_has_alias (child))
+		{
+			GbBookmark *safe_child = gb_bookmark_get_parented_alias_not_under (child, unsafe);
+			if (safe_child)
+			{
+				DEBUG_MSG (("Found a safe calias of %s under %s\n", 
+					    child->name, GB_BOOKMARK (safe_child->parent)->name));
+				gb_bookmark_alias_make_real (safe_child);
+			}
+			else 
+			{
+				DEBUG_MSG (("No safe parent found for %s\n", child->name));
+				if (GB_IS_FOLDER (child))
+				{
+					gb_folder_save_descendants (GB_FOLDER (child), unsafe);
+				}
+			}
+		}
+		else if (!gb_bookmark_is_alias (child) && GB_IS_FOLDER (child))
+		{
+			g_assert (!gb_bookmark_has_alias (child));
+			gb_folder_save_descendants (GB_FOLDER (child), unsafe);
+		}
+	}
+}
+
+void
+gb_bookmark_unparent_safe (GbBookmark *b)
+{
+	if (!gb_bookmark_is_alias (b) && gb_bookmark_has_alias (b))
+	{
+		GbBookmark *parented_alias = NULL;
+		if (GB_IS_FOLDER (b))
+		{
+			parented_alias = gb_bookmark_get_parented_alias_not_under (b, GB_FOLDER (b));
+		}
+		else
+		{
+			parented_alias = gb_bookmark_get_parented_alias (b);
+		}
+
+		if (parented_alias)
+		{
+			DEBUG_MSG (("safe parent for unparented bm: %s\n", GB_BOOKMARK (parented_alias->parent)->name));
+			gb_bookmark_alias_make_real (parented_alias);
+		}
+		else
+		{
+			DEBUG_MSG (("no safe parent found\n"));
+			if (GB_IS_FOLDER (b))
+			{
+				gb_folder_save_descendants (GB_FOLDER (b), GB_FOLDER (b));
+			}
+		}
+	}
+
+	g_assert (gb_bookmark_is_alias (b) 
+		  || !gb_bookmark_has_alias (b)
+		  || !b->alias->parent
+		  || (GB_IS_FOLDER (b) && gb_folder_is_ancestor (GB_FOLDER (b), b->alias)));
+
+	gb_bookmark_unparent (b);
+}
+
 static void
 gb_bookmark_unparent_internal (GbBookmark *b, gboolean emit_signals)
 {
@@ -766,6 +910,13 @@ gb_bookmark_unparent_internal (GbBookmark *b, gboolean emit_signals)
 		g_assert (((GbBookmark *) p)->alias_of == NULL);
 		
 		gb_bookmark_set_needs_saving (p);
+		if (GB_BOOKMARK (p)->set) gb_bookmark_set_tree_changed (GB_BOOKMARK (p)->set);
+		
+		/* it will be relinked when saving, if necessary */
+		if (b->xbel_node)
+		{
+			xmlUnlinkNode (b->xbel_node);
+		}
 		
 		if (GB_IS_FOLDER (b))
 		{
@@ -820,10 +971,174 @@ GbBookmark *
 gb_bookmark_real_bookmark (GbBookmark *b)
 {
 	while (b->alias_of)
+	{
 		b = b->alias_of;
+	}
 	return b;
 }
 
+GbBookmark *
+gb_bookmark_add_alias_under (GbBookmark *b, GbFolder *newparent)
+{
+	GbBookmark *real = gb_bookmark_real_bookmark (b);
+	if (real->parent || ((GbBookmark *) real->set->root) == real)
+	{
+		GbAliasPlaceholder *ap = gb_alias_placeholder_new (b->set, b->id);
+		GbBookmark *new = gb_bookmark_alias_create (b, ap);
+		gb_folder_add_child (newparent, new, -1);
+		g_object_unref (new);
+		g_object_unref (ap);
+		
+		g_assert (GB_IS_BOOKMARK (new));
+
+		return new;
+	}
+	else
+	{
+		gb_folder_add_child (newparent, real, -1);
+		return real;
+	}
+}
+
+GbBookmark *
+gb_bookmark_ensure_alias_under (GbBookmark *b, GbFolder *newparent)
+{
+	GbBookmark *new = gb_folder_get_child_or_alias (newparent, b);
+	if (!new)
+	{
+		new = gb_bookmark_add_alias_under (b, newparent);
+	}
+	return new;
+}
+
+GSList *
+gb_bookmark_get_all_alias_parents (GbBookmark *b)
+{
+	GSList *ret = NULL;
+	for (b = gb_bookmark_real_bookmark (b); b; b = b->alias)
+	{
+		if (b->parent)
+		{
+			ret = g_slist_prepend (ret, b->parent);
+		}
+	}
+	return ret;
+}
+
+/**
+ * Makes an alias the real bookmark of its chain. To be called before
+ * deleting the real bookmark.
+ */
+void
+gb_bookmark_alias_make_real (GbBookmark *alias)
+{
+	GbBookmark *real;
+	GSList *children_rev = NULL;
+	GSList *li;
+	GbBookmark *prevalias;
+	GbBookmark *nextalias;
+	xmlNodePtr alias_node;
+	xmlNodePtr real_node;
+
+	g_return_if_fail (gb_bookmark_is_alias (alias));
+	
+	real = gb_bookmark_real_bookmark (alias);
+
+	alias_node = alias->xbel_node;
+	if (alias_node)
+	{
+		xmlUnlinkNode (alias_node);
+	}
+	real_node = real->xbel_node;
+	if (real_node)
+	{
+		xmlUnlinkNode (real_node);
+	}
+
+	if (GB_IS_FOLDER (real))
+	{
+		g_return_if_fail (!gb_folder_is_ancestor (GB_FOLDER (real), alias));
+
+		children_rev = gb_folder_list_children_reversed (GB_FOLDER (real));
+		for (li = children_rev; li; li = li->next)
+		{
+			g_object_ref (li->data);
+			gb_bookmark_unparent (li->data);
+		}
+	}
+	
+	if (real->set && real->id)
+	{
+		g_hash_table_remove (real->set->id_to_bookmark, real->id);
+	}
+
+#ifdef NICK_HASHTABLE
+	if (real->set)
+	{
+		g_hash_table_remove (real->set->nick_to_bookmark, real->nick);
+	}
+#endif
+
+	if (GB_IS_SITE (real) && real->set)
+	{
+		g_hash_table_remove (real->set->url_to_bookmark, GB_SITE (real)->url);
+	}
+
+	prevalias = alias->alias_of;
+	nextalias = alias->alias;
+
+	g_assert (prevalias->alias == alias);
+	g_assert (!nextalias || nextalias->alias_of == alias);
+	
+	prevalias->alias = nextalias;
+	if (nextalias)
+	{
+		nextalias->alias_of = prevalias;
+	}
+	
+	alias->alias_of = NULL;
+	alias->alias = real;
+	g_assert (real->alias_of == NULL);
+	real->alias_of = alias;
+
+	if (alias->set && alias->id)
+	{
+		g_hash_table_insert (alias->set->id_to_bookmark, alias->id, alias);
+	}
+
+#ifdef NICK_HASHTABLE
+	if (alias->set)
+	{
+		g_hash_table_insert (alias->set->nick_to_bookmark, alias->nick);
+	}
+#endif
+
+	if (GB_IS_SITE (alias) && alias->set)
+	{
+		g_hash_table_insert (alias->set->url_to_bookmark, GB_SITE (alias)->url, alias);
+	}
+
+	if (GB_IS_FOLDER (alias))
+	{
+		for (li = children_rev; li; li = li->next)
+		{
+			gb_folder_add_child (GB_FOLDER (alias), li->data, 0);
+			g_object_unref (li->data);
+		}
+	}
+	g_slist_free (children_rev);
+
+	gb_bookmark_set_xbel_node (alias, real_node);
+	gb_bookmark_set_xbel_node (real, alias_node);
+
+	gb_bookmark_set_needs_saving (alias);
+
+	do {
+		g_signal_emit (alias, GbBookmarkSignals[GB_BOOKMARK_MODIFIED], 0);
+		gb_folder_emit_child_modified (alias->parent, alias);
+	} while ((alias = alias->alias) != NULL);
+}
+
 
 /**
  * Site object
@@ -1315,8 +1630,15 @@ gb_smart_site_subst_args (GbSmartSite *b, gchar **args)
 	for (i = 0; i < nargs; i++)
 	{
 		gchar *arg = g_convert (args[i], strlen (args[i]), encoding, "UTF-8", NULL, NULL, NULL);
-		if (!arg) arg = g_strdup (args[i]);
-		encoded_args[i] = arg;
+		if (arg)
+		{
+			encoded_args[i] = gnome_vfs_escape_string (arg);
+			g_free (arg);
+		}
+		else
+		{
+			encoded_args[i] = gnome_vfs_escape_string (args[i]);
+		}
 	}
 	encoded_args[nargs] = NULL;
 
@@ -1343,8 +1665,8 @@ gb_smart_site_subst_args (GbSmartSite *b, gchar **args)
 		}
 	}
 
-	/* there may (shoud) be still a %s, the last one. Put all the remaining args in it, 
-	   separated with spaces */
+	/* there may (should) be still a %s, the last one. Put all the
+	   remaining args in it, separated with (url-encoded) spaces */
 	t2 = strstr (t1, "%s");
 	if (t2) 
 	{
@@ -1355,7 +1677,7 @@ gb_smart_site_subst_args (GbSmartSite *b, gchar **args)
 			args++;
 			if (*args != NULL)
 			{
-				g_string_append_c (s, ' ');
+				g_string_append (s, "%20");
 			}
 		}
 		t1 = t2 + 2;
@@ -1835,6 +2157,14 @@ gb_folder_add_child (GbFolder *p, GbBookmark *c, gint position)
 	g_return_if_fail (GB_IS_BOOKMARK (c));
 	g_return_if_fail (position >= -1);
 	g_return_if_fail (GB_BOOKMARK (p)->set);
+	g_return_if_fail (GB_BOOKMARK (p) != c);
+
+	if (gb_bookmark_is_alias (c)
+	    && gb_bookmark_real_bookmark (c)->parent == NULL 
+	    && gb_bookmark_real_bookmark (c) != GB_BOOKMARK (gb_bookmark_real_bookmark (c)->set->root))
+	{
+		g_warning ("Adding an alias whose real bookmark is unparented.");
+	}
 	
 	p = GB_FOLDER (gb_bookmark_real_bookmark ((GbBookmark *) p));
 
@@ -1851,6 +2181,8 @@ gb_folder_add_child (GbFolder *p, GbBookmark *c, gint position)
 		gb_bookmark_set_set (c, GB_BOOKMARK (p)->set);
 	}
 	
+	gb_bookmark_set_tree_changed (GB_BOOKMARK (p)->set);
+
 	cprev = NULL;
 	cnext = p->child;
 	i = 0;
@@ -2072,8 +2404,8 @@ gb_folder_is_default_folder (GbFolder *f)
 	return (GB_BOOKMARK (rf)->set) && (rf == GB_BOOKMARK (rf)->set->default_folder);
 }
 
-GSList *
-gb_folder_list_children (GbFolder *f)
+static GSList *
+gb_folder_list_children_reversed (GbFolder *f)
 {
 	GSList *l = NULL;
 	GbBookmark *bi;
@@ -2083,7 +2415,13 @@ gb_folder_list_children (GbFolder *f)
 		l = g_slist_prepend (l, bi);
 	}
 
-	return g_slist_reverse (l);
+	return l;
+}
+
+GSList *
+gb_folder_list_children (GbFolder *f)
+{
+	return g_slist_reverse (gb_folder_list_children_reversed (f));
 }
 
 static gint 
@@ -2131,6 +2469,48 @@ gb_folder_sort (GbFolder *f, gboolean folders_first, gboolean recursive)
 	g_slist_free (l);
 }
 
+gboolean 
+gb_folder_has_child_or_alias (GbFolder *f, GbBookmark *c)
+{
+	f = GB_FOLDER (gb_bookmark_real_bookmark (GB_BOOKMARK (f)));
+	for (c = gb_bookmark_real_bookmark (c); c; c= c->alias)
+	{
+		if (c->parent == f)
+		{
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
+static GbBookmark *
+gb_folder_get_child_or_alias (GbFolder *f, GbBookmark *c)
+{
+	GbBookmark *i;
+	f = GB_FOLDER (gb_bookmark_real_bookmark (GB_BOOKMARK (f)));
+	c = gb_bookmark_real_bookmark (c);
+	for (i = f->child; i; i = i->next)
+	{
+		if (gb_bookmark_real_bookmark (i) == c)
+		{
+			return i;
+		}
+	}
+	return NULL;
+}
+
+void
+gb_folder_remove_child_or_aliases (GbFolder *f, GbBookmark *c)
+{
+	GbBookmark *v;
+	g_object_ref (c);
+	while ((v = gb_folder_get_child_or_alias (f, c)) != NULL)
+	{
+		gb_bookmark_unparent_safe (v);
+	}
+	g_object_unref (c);
+}
+
 /**
  * Separator object
  */
@@ -2262,12 +2642,12 @@ MAKE_GET_TYPE (gb_alias_placeholder, "GbAliasPlaceholder", GbAliasPlaceholder,
 	       gb_alias_placeholder_class_init, gb_alias_placeholder_init, GB_TYPE_BOOKMARK);
 
 GbAliasPlaceholder *
-gb_alias_placeholder_new (GbBookmarkSet *set, gulong alias_of_id)
+gb_alias_placeholder_new (GbBookmarkSet *set, const char *alias_of_id)
 {
 	GbAliasPlaceholder *b;
 	b = g_object_new (GB_TYPE_ALIAS_PLACEHOLDER, NULL);
 	gb_bookmark_set_set (GB_BOOKMARK (b), set);
-	b->alias_of_id = alias_of_id;
+	b->alias_of_id = g_strdup (alias_of_id);
 	return b;
 }
 
@@ -2277,6 +2657,7 @@ gb_alias_placeholder_class_init (GbAliasPlaceholderClass *klass)
 	klass->parent_class.gb_bookmark_copy = gb_alias_placeholder_copy_impl;
 	klass->parent_class.gb_bookmark_set_set = gb_alias_placeholder_set_set_impl;
 	klass->parent_class.gb_bookmark_alias_create = gb_alias_placeholder_alias_create_impl;
+	klass->parent_class.gb_bookmark_create_toolbar_widget = gb_alias_placeholder_create_toolbar_widget_impl;
 	G_OBJECT_CLASS (klass)->finalize = gb_alias_placeholder_finalize_impl;
 
 	gb_alias_placeholder_class = klass;
@@ -2295,10 +2676,16 @@ gb_alias_placeholder_copy_impl (GbBookmark *b)
 static void
 gb_alias_placeholder_copy_impl_do (GbAliasPlaceholder *b, GbAliasPlaceholder *c)
 {
-	c->alias_of_id = b->alias_of_id;
+	c->alias_of_id = g_strdup (b->alias_of_id);
 	c->create_toolbar = b->create_toolbar;
 }
 
+static GbTbWidget *
+gb_alias_placeholder_create_toolbar_widget_impl (GbBookmark *b)
+{
+	g_return_val_if_fail (GB_IS_ALIAS_PLACEHOLDER (b), NULL);
+	return gb_create_toolbar_widget_alias_placeholder (GB_ALIAS_PLACEHOLDER (b));
+}
 
 static void
 gb_alias_placeholder_set_set_impl (GbBookmark *b, GbBookmarkSet *set)
@@ -2369,15 +2756,26 @@ gb_alias_placeholder_resolve (GbAliasPlaceholder *ap)
 	g_return_val_if_fail (GB_IS_BOOKMARK_SET (set), FALSE);
 	g_return_val_if_fail (GB_BOOKMARK (ap)->parent, FALSE);
 
-	real = g_hash_table_lookup (set->id_to_bookmark, GINT_TO_POINTER (ap->alias_of_id));
-	if (!real) return FALSE;
+	if (ap->alias_of_id)
+	{
+		real = g_hash_table_lookup (set->id_to_bookmark, ap->alias_of_id);
+		if (!real) return FALSE;
+	}
+	else
+	{
+		return FALSE;
+	}
 
 	alias = gb_bookmark_alias_create (real, ap);
 	if (!GB_IS_BOOKMARK (alias)) return FALSE;
 
 	pos = gb_folder_get_child_index (((GbBookmark *) ap)->parent, (GbBookmark *) ap);
 	gb_folder_add_child (((GbBookmark *) ap)->parent, alias, pos);
-	g_object_unref (G_OBJECT (alias));
+	g_object_unref (alias);
+	
+	gb_bookmark_set_xbel_node (alias, GB_BOOKMARK (ap)->xbel_node);
+	gb_bookmark_set_xbel_node (GB_BOOKMARK (ap), NULL);
+
 	gb_bookmark_unparent ((GbBookmark *) ap);
 	set->unresolved_aliases = g_slist_remove (set->unresolved_aliases, ap);
 
@@ -2433,9 +2831,10 @@ gb_bookmark_set_init (GbBookmarkSet *b)
 	b->file_format_version = GB_FILE_FORMAT_VERSION_GALEON_2;
 	b->filename = NULL;
 	b->io = NULL;
+	b->xbel_doc = NULL;
 	b->needs_saving = FALSE;
 	b->default_folder = b->root;
-	b->id_to_bookmark = g_hash_table_new (NULL, NULL);
+	b->id_to_bookmark = g_hash_table_new (g_str_hash, g_str_equal);
 	b->url_to_bookmark = g_hash_table_new (g_str_hash, g_str_equal);
 #ifdef NICK_HASHTABLE
 	b->nick_to_bookmark = g_hash_table_new (g_str_hash, g_str_equal);
@@ -2477,6 +2876,8 @@ gb_bookmark_set_finalize_impl (GObject *o)
 
 	DEBUG_MSG (("in gb_bookmark_set_finalize_impl\n"));
 
+	gb_bookmark_set_set_xbel_doc (set, NULL);
+	
 	if (set->autosave_timeout_id != 0)
 	{
 		g_source_remove (set->autosave_timeout_id);
@@ -2486,7 +2887,7 @@ gb_bookmark_set_finalize_impl (GObject *o)
 	{
 		g_object_unref (G_OBJECT (set->io));
 	}
-
+	
 	if (set->default_folder)
 	{
 		g_object_unref (G_OBJECT (set->default_folder));
@@ -2514,6 +2915,8 @@ gb_bookmark_set_finalize_impl (GObject *o)
 
 	g_slist_free (set->unresolved_aliases);
 	g_slist_free (set->toolbars);
+	g_slist_free (set->all_bookmarks_noalias);
+	g_slist_free (set->all_bookmarks);
 
 	G_OBJECT_CLASS (g_object_class)->finalize (o);
 }
@@ -2649,6 +3052,32 @@ gb_bookmark_set_set_io (GbBookmarkSet *set, GbIO *io)
 	set->io = io;
 }
 
+void
+gb_bookmark_set_discard_xml_nodes (GbBookmarkSet *set)
+{
+	GbIterator *i = gb_iterator_new (set);
+	while (gb_iterator_has_next (i))
+	{
+		GbBookmark *b = gb_iterator_next (i);
+		gb_bookmark_set_xbel_node (b, NULL);
+	}
+	g_object_unref (i);
+}
+
+void
+gb_bookmark_set_set_xbel_doc (GbBookmarkSet *set, xmlDocPtr doc)
+{
+	g_return_if_fail (GB_IS_BOOKMARK_SET (set));
+
+	if (set->xbel_doc && set->xbel_doc != doc)
+	{
+		gb_bookmark_set_discard_xml_nodes (set);
+		xmlFreeDoc (set->xbel_doc);
+	}
+
+	set->xbel_doc = doc;
+}
+
 gboolean
 gb_bookmark_set_check_save (GbBookmarkSet *set)
 {
@@ -2667,6 +3096,7 @@ gb_bookmark_set_check_save (GbBookmarkSet *set)
 		}
 		else
 		{
+			g_warning ("Not saving the bookmarks because either the filename or the IO has not been set.");
 			return FALSE;
 		}
 	}
@@ -2932,3 +3362,74 @@ gb_bookmark_set_emit_autocompletion_source_data_changed (GbBookmarkSet *gh)
 	g_signal_emit_by_name (gh, "data-changed");
 }
 
+static GSList *
+gb_folder_prepend_preorder_noalias (GbFolder *f, GSList *l)
+{
+	GbBookmark *c;
+	for (c = f->child; c; c = c->next)
+	{
+		if (!gb_bookmark_is_alias (c))
+		{
+			l = g_slist_prepend (l, c);
+			if (GB_IS_FOLDER (c))
+			{
+				l = gb_folder_prepend_preorder_noalias (GB_FOLDER (c), l);
+			}
+		}
+	}
+	return l;
+}
+
+static GSList *
+gb_folder_prepend_preorder (GbFolder *f, GSList *l)
+{
+	GbBookmark *c;
+	for (c = f->child; c; c = c->next)
+	{
+		l = g_slist_prepend (l, c);
+		if (!gb_bookmark_is_alias (c) &&GB_IS_FOLDER (c))
+		{
+			l = gb_folder_prepend_preorder (GB_FOLDER (c), l);
+		}
+	}
+	return l;
+}
+
+const GSList *
+gb_bookmark_set_get_all (GbBookmarkSet *set)
+{
+	if (!set->all_bookmarks)
+	{
+		set->all_bookmarks = g_slist_reverse 
+			(gb_folder_prepend_preorder (set->root, 
+						     g_slist_prepend (NULL, set->root)));
+	}
+	return set->all_bookmarks;
+}
+
+const GSList *
+gb_bookmark_set_get_all_noalias (GbBookmarkSet *set)
+{
+	if (!set->all_bookmarks_noalias)
+	{
+		set->all_bookmarks_noalias = g_slist_reverse 
+			(gb_folder_prepend_preorder_noalias (set->root, 
+							     g_slist_prepend (NULL, set->root)));
+	}
+	return set->all_bookmarks_noalias;
+}
+
+static void
+gb_bookmark_set_tree_changed (GbBookmarkSet *set)
+{
+	if (set->all_bookmarks_noalias)
+	{
+		g_slist_free (set->all_bookmarks_noalias);
+		set->all_bookmarks_noalias = NULL;
+	}
+	if (set->all_bookmarks)
+	{
+		g_slist_free (set->all_bookmarks);
+		set->all_bookmarks = NULL;
+	}
+}
