@@ -222,7 +222,7 @@ nsresult GaleonWrapper::PrintPreviewClose (void)
 	if (NS_FAILED(rv) || !print) return NS_ERROR_FAILURE;
 
 	rv = print->GetDoingPrintPreview(&isPreview);
-	if (isPreview == PR_TRUE)
+	if (NS_SUCCEEDED (rv) && isPreview == PR_TRUE)
 	{
 		rv = print->ExitPrintPreview();
 	}
@@ -367,10 +367,11 @@ nsresult GaleonWrapper::SetZoomOnDocshell (float aZoom, nsIDocShell *DocShell)
 
 	nsCOMPtr<nsIPresContext> PresContext;
 	result = DocShell->GetPresContext (getter_AddRefs(PresContext));
-	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+	if (NS_FAILED(result) || !PresContext) return NS_ERROR_FAILURE;
 					
 	nsCOMPtr<nsIDeviceContext> DeviceContext;
 	result = PresContext->GetDeviceContext (getter_AddRefs(DeviceContext));
+	if (NS_FAILED(result) || !DeviceContext) return NS_ERROR_FAILURE;
 
 	return DeviceContext->SetTextZoom (aZoom);
 }
@@ -690,6 +691,46 @@ nsresult GaleonWrapper::GetDocumentTitle (char **title)
 	return NS_OK;
 }
 
+
+nsresult  GaleonWrapper::ClearHistory ()
+{
+	nsresult result;
+	PRInt32 count, index;
+	nsCOMPtr<nsIHistoryEntry> he;
+	nsCOMPtr<nsISHEntry> she;
+
+	nsCOMPtr<nsISHistory> SessionHistory;
+	result = GetSHistory (getter_AddRefs(SessionHistory));
+	if (NS_FAILED(result) || !SessionHistory) return NS_ERROR_FAILURE;
+
+	SessionHistory->GetCount (&count);
+	if (count <= 1) return NS_OK;
+
+	SessionHistory->GetIndex (&index);
+	
+	/* Save the current entry */
+	result = SessionHistory->GetEntryAtIndex (index, PR_FALSE,
+						  getter_AddRefs (he));
+	if (!NS_SUCCEEDED(result) || (!he))
+		return NS_ERROR_FAILURE;
+
+	she = do_QueryInterface (he);
+	if (!she) return NS_ERROR_FAILURE;
+
+	/* Clear the history */
+	SessionHistory->PurgeHistory (count);
+
+	/* Put the current entry back in */
+	nsCOMPtr<nsISHistoryInternal> hi = do_QueryInterface (SessionHistory);
+	if (!hi) return NS_ERROR_FAILURE;
+
+	result = hi->AddEntry (she, PR_TRUE);
+	if (!NS_SUCCEEDED(result) || (!she))
+		return NS_ERROR_FAILURE;
+
+	return NS_OK;
+}
+
 nsresult  GaleonWrapper::CopyHistoryTo (GaleonWrapper *dest, 
 					PRBool back_history, 
 					PRBool forward_history, 
