@@ -59,7 +59,7 @@
 #else
 #include <nsICharsetConverterManager2.h>
 #endif
-#include <nsIFontList.h>
+#include <nsIFontEnumerator.h>
 #include <nsISupportsPrimitives.h>
 #include <nsReadableUtils.h>
 #include <nsIPermissionManager.h>
@@ -71,6 +71,7 @@
 #include <nsCCookieManager.h>
 #include <nsCPasswordManager.h>
 #include <nsIObserverService.h>
+#include <nsIPrefService.h>
 
 //For setting the locale. hopefully gtkmozembed will do itself soon
 #include <chrome/nsIChromeRegistry.h>
@@ -80,6 +81,11 @@
 #define MOZILLA_PROFILE_NAME "galeon"
 #define MOZILLA_PROFILE_FILE "prefs.js"
 
+//Absolute path to the file holding default preferences
+#ifndef DEFAULT_PROFILE_FILE
+#define DEFAULT_PROFILE_FILE SHARE_DIR"/default-prefs.js"
+#endif
+
 #if MOZILLA_SNAPSHOT > 9
 //Table of permission types to map to new string based api
 const char* gPermType[3] = { "cookie", "image", "popup" };
@@ -324,36 +330,47 @@ mozilla_load_proxy_prefs (MozillaEmbedShell *shell)
 static void
 mozilla_set_default_prefs (void)
 {
-        mozilla_prefs_set_boolean ("mozilla.widget.raise-on-setfocus",
-                                   FALSE);
-        mozilla_prefs_set_boolean ("browser.display.use_system_colors",
-                                   FALSE);
+	nsCOMPtr<nsIPrefService> prefs
+		= do_GetService (NS_PREFSERVICE_CONTRACTID);
+	g_return_if_fail (prefs);
 
-        /* set default search engine */
-        mozilla_prefs_set_string ("keyword.URL",_("http://www.google.com/search?q="));
-        mozilla_prefs_set_boolean ("keyword.enabled", FALSE);
-        mozilla_prefs_set_boolean ("security.checkloaduri", FALSE); 
+	// read our predefined default prefs
+	nsresult rv;
+	nsCOMPtr<nsILocalFile> file;
+	rv = NS_NewNativeLocalFile(
+		NS_LITERAL_CSTRING(DEFAULT_PROFILE_FILE),
+		PR_TRUE, getter_AddRefs(file));
+	if (NS_SUCCEEDED(rv))
+	{
+		rv = prefs->ReadUserPrefs (file);
+	}
 
-        /* while we have no UI */
-        mozilla_prefs_set_boolean ("wallet.captureForms", FALSE);
+	if (NS_FAILED(rv))
+	{
+		g_warning ("failed to read default preferences, error: %x", rv);
+	}
 
-        /* deactivate mailcap and mime.types support */
-        mozilla_prefs_set_string ("helpers.global_mime_types_file", "");
-        mozilla_prefs_set_string ("helpers.global_mailcap_file", "");
-        mozilla_prefs_set_string ("helpers.private_mime_types_file", "");
-        mozilla_prefs_set_string ("helpers.private_mailcap_file", "");
-        
-        /* dont allow xpi installs from galeon, there are crashes */
-        mozilla_prefs_set_boolean ("xpinstall.enabled", FALSE);
+	// Load the default user preferences as well.  This also makes the
+	// prefs to be saved in the user's prefs.js file, instead of messing up
+	// our global defaults file.
+	rv = prefs->ReadUserPrefs (nsnull);
+	if (NS_FAILED(rv)) 
+	{
+		g_warning ("failed to read user preferences, error: %x", rv);
+	}
 
-        /* disable sucky XUL ftp view, have nice ns4-like html page instead */
-        mozilla_prefs_set_boolean ("network.dir.generate_html", TRUE);
-        
-        /* set the right accept encoding flags */
-        mozilla_prefs_set_string ("network.http.accept-encoding" ,
-                                  "gzip, deflate, compress;q=0.9");
-	
-        mozilla_prefs_save ();
+	nsCOMPtr<nsIPrefBranch> pref;
+	prefs->GetDefaultBranch ("", getter_AddRefs(pref));
+	g_return_if_fail (pref);
+
+	// Search engine for keyword search, although disabled by default, set
+	// here to facilitate l10n.
+	pref->SetCharPref ("keyword.URL", _("http://www.google.com/search?q="));
+
+	// Also set vendor and vendorSub prefs so that mozilla can construct
+	// the user agent string for us.
+	pref->SetCharPref ("general.useragent.vendor", "Galeon");
+	pref->SetCharPref ("general.useragent.vendorSub", VERSION);
 }
 
 static void
@@ -561,13 +578,9 @@ mozilla_embed_shell_init (MozillaEmbedShell *mes)
 	mozilla_init_chrome();
 
 	/* Post initialization */
+	mozilla_set_default_prefs ();
 	mozilla_notifiers_init (mes);
 
-	if (new_build)
-	{
-		mozilla_set_default_prefs ();
-	}
-
 	mozilla_load_proxy_prefs (mes);
 	
 	mozilla_init_single (mes);
@@ -650,6 +663,16 @@ mozilla_embed_shell_finalize (GObject *object)
 	
 	gtk_moz_embed_pop_startup ();
 	
+	if (mes->priv->charsets_hash)
+	{
+		g_hash_table_destroy (mes->priv->charsets_hash);
+	}
+
+	if (mes->priv->sorted_charsets_titles)
+	{
+		g_list_free (mes->priv->sorted_charsets_titles);
+	}
+
         g_free (mes->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -767,7 +790,7 @@ fill_charsets_lists (MozillaEmbedShellPrivate *priv)
 	if (NS_FAILED(rv)) return G_FAILED;
 
         PRUint32 translated_cscount = get_translated_cscount ();
-        priv->charsets_hash = g_hash_table_new (g_str_hash, g_str_equal);
+        priv->charsets_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
 	PRBool more;
 	for (rv = e->HasMore(&more); more; rv = e->HasMore(&more))
@@ -790,7 +813,7 @@ fill_charsets_lists (MozillaEmbedShellPrivate *priv)
                                 charset_trans_array[j].charset_name) == 0)
                         {
                                 g_free (charset_title_str);
-                                charset_title_str = (char *) 
+                                charset_title_str = g_strdup (_(charset_trans_array[j].charset_title));
                                         _(charset_trans_array[j].charset_title);
                                 break;
                         }
@@ -827,7 +850,7 @@ fill_charsets_lists (MozillaEmbedShellPrivate *priv)
         if (!NS_SUCCEEDED(rv)) return G_FAILED;
 
         rv = cs_list->Count(&cscount);
-        priv->charsets_hash = g_hash_table_new (g_str_hash, g_str_equal);
+        priv->charsets_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 	for (PRUint32 i = 0; i < cscount; i++)
         {
                 nsCOMPtr<nsISupports> cssupports =
@@ -869,8 +892,7 @@ fill_charsets_lists (MozillaEmbedShellPrivate *priv)
                                 charset_trans_array[j].charset_name) == 0)
                         {
                                 g_free (charset_title_str);
-                                charset_title_str = (char *) 
-                                        _(charset_trans_array[j].charset_title);
+                                charset_title_str = g_strdup (_(charset_trans_array[j].charset_title));
                                 break;
                         }
                 }
@@ -956,34 +978,29 @@ impl_get_font_list (GaleonEmbedShell *shell,
 {
 	nsresult rv;
 
-	nsCOMPtr<nsIFontList> mozFontList;
-	mozFontList = do_CreateInstance("@mozilla.org/gfx/fontlist;1", &rv);
-	if(NS_FAILED(rv)) return G_FAILED;
+	nsCOMPtr<nsIFontEnumerator> mozFontEnumerator;
+	mozFontEnumerator = do_CreateInstance("@mozilla.org/gfx/fontenumerator;1", &rv);
+	NS_ENSURE_SUCCESS(rv, G_FAILED);
 
-	nsCOMPtr<nsISimpleEnumerator> fontEnum;
-	mozFontList->AvailableFonts(NS_ConvertUTF8toUCS2(langGroup).get(),
-				    NS_ConvertUTF8toUCS2(fontType).get(),
-				    getter_AddRefs(fontEnum));
-	if(NS_FAILED(rv)) return G_FAILED;
+	PRUint32    n_fonts;
+	PRUnichar **fonts;
+	rv = mozFontEnumerator->EnumerateFonts (langGroup, fontType,
+			                        &n_fonts, &fonts);
+	NS_ENSURE_SUCCESS(rv, G_FAILED);
 
 	GList *l = NULL;
-	PRBool enumResult;
-	for(fontEnum->HasMoreElements(&enumResult) ;
-	    enumResult == PR_TRUE;
-	    fontEnum->HasMoreElements(&enumResult))
+	for (PRUint32 i = 0; i < n_fonts; i++)
 	{
-		nsCOMPtr<nsISupportsString> fontName;
-		fontEnum->GetNext(getter_AddRefs(fontName));
-		if(NS_FAILED(rv)) return G_FAILED;
+		char *str;
 
-		nsString fontString;
-		fontName->GetData(fontString);
-
-		char *gFontString;
-		gFontString = g_strdup(NS_ConvertUCS2toUTF8(fontString).get());
-		l = g_list_append(l, gFontString);
+		str = g_strdup (NS_ConvertUCS2toUTF8(fonts[i]).get());
+		l = g_list_prepend (l, str);
+		nsMemory::Free (fonts[i]);
 	}
-	*fontList = l;
+
+	nsMemory::Free (fonts);
+
+	*fontList = g_list_reverse (l);
 
 	if (default_font != NULL)
 	{
