@@ -22,9 +22,13 @@
 
 #include "galeon-window.h"
 #include "galeon-sidebar.h"
+#include "galeon-dnd.h"
 #include "gul-state.h"
 #include "gul-gobject-misc.h"
 #include "gul-notebook.h"
+#include "gul-toolbar.h"
+#include "gul-gui.h"
+#include "gul-general.h"
 #include "statusbar.h"
 #include "toolbar.h"
 #include "ppview-toolbar.h"
@@ -43,6 +47,7 @@
 #include "bookmarks-editor-dockable.h"
 #include "window-recent-history-menu.h"
 #include "galeon-embed-prefs.h"
+#include "galeon-css-menu.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -51,13 +56,33 @@
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-ui-component.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
-#include <gtk/gtk.h>
+#include <libgnome/gnome-config.h>
 #include <X11/X.h>
 #include <X11/Xlib.h>
 #include <gdk/gdkx.h>
 #include <gdk/gdkkeysyms.h>
+#include <gtk/gtkmenuitem.h>
+#include <gtk/gtkpaned.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkentry.h>
+#include <gtk/gtkeditable.h>
+#include <gtk/gtkdialog.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkhpaned.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtkmessagedialog.h>
 
+/* keys and defaults for state */
+#define STATE_SIDEBAR_SIZE         "Main Window/sidebar_size"
+
+#define STATE_SIDEBAR_SIZE_DEFAULT 175
+
+#define STATE_MAIN_WIDTH_DEFAULT   700
+#define STATE_MAIN_HEIGHT_DEFAULT  500
+
+/* bonoboui paths */
 #define CHARSET_MENU_PATH "/menu/View/EncodingMenuPlaceholder"
+#define CSS_MENU_PATH "/menu/View/AlternateCssPlaceholder"
 #define GO_RECENT_HISTORY_PATH "/menu/Go/RecentHistory"
 
 #define GO_BACK_CMD_PATH "/commands/GoBack"
@@ -171,6 +196,7 @@ static BonoboUIVerb galeon_popup_verbs [] = {
 	BONOBO_UI_VERB ("EPOpenImageInNewWindow", (BonoboUIVerbFn)popup_cmd_image_in_new_window),
 	BONOBO_UI_VERB ("EPOpenImageInNewTab", (BonoboUIVerbFn)popup_cmd_image_in_new_tab),
 	BONOBO_UI_VERB ("EPBlockImageSite", (BonoboUIVerbFn)popup_cmd_image_block_site),
+	BONOBO_UI_VERB ("EPAllowImageSite", (BonoboUIVerbFn)popup_cmd_image_block_site),
 	BONOBO_UI_VERB ("DPOpenFrameInNewWindow", (BonoboUIVerbFn)popup_cmd_frame_in_new_window),
 	BONOBO_UI_VERB ("DPOpenFrameInNewTab", (BonoboUIVerbFn)popup_cmd_frame_in_new_tab),
 	BONOBO_UI_VERB ("DPAddFrameBookmark", (BonoboUIVerbFn)popup_cmd_add_frame_bookmark),
@@ -181,12 +207,14 @@ static BonoboUIVerb galeon_popup_verbs [] = {
 
 struct GaleonWindowPrivate
 {
+	GList *notifiers;
 	PPViewToolbar *ppview_toolbar;
 	Toolbar *toolbar;
 	Statusbar *statusbar;
 	GbBonoboUIMenu *bookmarks_menu;
 	GbBonoboUIMenu *bookmarks_context_menu;
 	GbBonoboUIToolbar *bookmarks_toolbar;
+	GaleonCssMenu *css_menu;
 	GtkNotebook *notebook;
 	GtkWidget *hpaned;
 	GaleonTab *active_tab;
@@ -228,6 +256,19 @@ static void
 galeon_window_tab_detached_cb    (GulNotebook *notebook, gint page,
 				  gint x, gint y, GaleonWindow *window);
 
+static void             
+galeon_window_tab_drag_data_received_cb (GtkWidget* widget, GdkDragContext *context,
+					 gint x, gint y, GtkSelectionData *selection_data,
+					 guint info, guint time, GtkWidget *child);
+
+static void             
+galeon_window_tab_added_cb  (GulNotebook *notebook, GtkWidget* child, 
+			     GaleonWindow *window);
+
+static void             
+galeon_window_tab_removed_cb  (GulNotebook *notebook, GtkWidget* child, 
+			       GaleonWindow *window);
+
 void /* yes, this is not static */
 galeon_window_bookmark_activate (GaleonWindow *w, GbBookmarkEventActivated *ev);
 
@@ -240,10 +281,16 @@ static void
 galeon_window_recent_history_menu_activated_cb (GaleonWindowRecentHistoryMenu *wrhm, 
 						const char *url, const char *title, 
 						GaleonWindow *w);
+static void
+galeon_window_fullscreen (GaleonWindow *window);
+static void
+galeon_window_unfullscreen (GaleonWindow *window);
+
 
 /* static class variables */
 static GdkColor _galeon_window_loading_tab_color;
 static GdkColor _galeon_window_new_tab_color;
+static GdkPixbufAnimation * _galeon_window_loading_animation;
 
 static GObjectClass *parent_class = NULL;
 
@@ -272,6 +319,11 @@ galeon_window_class_init (GaleonWindowClass *klass)
 	tmp = eel_gconf_get_string (CONF_TABS_TABBED_NEW_COLOR);
 	gdk_color_parse (tmp, &_galeon_window_new_tab_color);
 	g_free (tmp);
+
+	tmp =  gul_general_user_file( "galeon-tab-loading.gif", FALSE );
+	_galeon_window_loading_animation = 
+		gdk_pixbuf_animation_new_from_file (tmp, NULL);
+	g_free (tmp);
 }
 
 static void
@@ -289,12 +341,10 @@ galeon_window_key_press_event_cb (GtkWidget *widget,
 {
         int page;
 
-	g_print("Window keypress\n");
-
-	if ((event->state & GDK_Shift_L) || (event->state & GDK_Shift_R))
+	if (event->state & GDK_SHIFT_MASK)
                 return FALSE;
 
-        if ((event->state & GDK_Alt_L) || (event->state & GDK_Alt_R))
+        if (event->state & GDK_MOD1_MASK)
         {
                 page = event->keyval - GDK_0 -1;
 
@@ -337,36 +387,33 @@ galeon_window_selection_received_cb (GtkWidget *widget,
 	}
 }
 	
-static void
-save_window_state (GtkWidget *win)
-{
-	GaleonWindow *window = GALEON_WINDOW (win);
-	
-	if (!(window->priv->chrome_mask & EMBED_CHROME_OPENASPOPUP) &&
-	    !(window->priv->chrome_mask & EMBED_CHROME_OPENASFULLSCREEN))
-	{
-		gul_state_save_window (win, "main_window");
-	}
-}
-
-static gboolean
-galeon_window_configure_event_cb (GtkWidget *widget,
-				  GdkEventConfigure *event,
-				  gpointer data)
-{	
-	save_window_state (widget);
-	return FALSE;
-}
-
 static gboolean
 galeon_window_state_event_cb (GtkWidget *widget,
 			      GdkEventWindowState *event,
 			      gpointer data)
 {
-	save_window_state (widget);
+	if (event->changed_mask & GDK_WINDOW_STATE_FULLSCREEN)
+	{
+		GaleonWindow *window = GALEON_WINDOW (widget);
+		if (event->new_window_state & GDK_WINDOW_STATE_FULLSCREEN)
+		{
+			galeon_window_fullscreen (window);
+		}
+		else
+		{
+			galeon_window_unfullscreen (window);
+		}
+	}
 	return FALSE;
 }
 
+static gboolean
+delete_event_cb (GaleonWindow *window)
+{
+	window->priv->closing = TRUE;
+	return gul_gui_widget_hide_now (GTK_WIDGET(window));
+}
+
 static void
 setup_bonobo_window (GaleonWindow *window,
 		     BonoboUIComponent **ui_component)
@@ -390,11 +437,7 @@ setup_bonobo_window (GaleonWindow *window,
 
 	bonobo_ui_util_set_ui (*ui_component,
 			       DATADIR,
-#ifdef ENABLE_SIMPLE_MENUBAR
-			       "galeon-simple-ui.xml",
-#else
 			       "galeon-ui.xml",
-#endif
 			       "galeon", NULL);
 
 	/* Key handling */
@@ -403,10 +446,10 @@ setup_bonobo_window (GaleonWindow *window,
                          G_CALLBACK(galeon_window_key_press_event_cb),
                          window);
 
-	g_signal_connect (G_OBJECT(win), "configure_event",
-			  G_CALLBACK (galeon_window_configure_event_cb), NULL);
 	g_signal_connect (G_OBJECT(win), "window_state_event",
 			  G_CALLBACK (galeon_window_state_event_cb), NULL);
+	g_signal_connect (G_OBJECT(win), "delete-event",
+			  G_CALLBACK (delete_event_cb), NULL);
 	
 	g_signal_connect (G_OBJECT(win),
 			  "selection-received",
@@ -449,6 +492,19 @@ tabbed_position_gconf_changed_cb(GConfClient *client,
 	gtk_notebook_set_tab_pos (notebook, position);
 }
 
+
+static void
+tabbed_always_show_gconf_changed_cb(GConfClient *client,
+				    guint cnxn_id,
+				    GConfEntry *entry,
+				    GtkNotebook *notebook)
+{
+	GtkPositionType policy = eel_gconf_get_boolean (CONF_TABS_TABBED_ALWAYS_SHOW)
+		? GTK_POLICY_ALWAYS
+		: GTK_POLICY_AUTOMATIC;
+	gul_notebook_set_policy (GUL_NOTEBOOK (notebook), policy);
+}
+
 static GtkNotebook *
 setup_notebook (GaleonWindow *window)
 {
@@ -476,9 +532,29 @@ setup_notebook (GaleonWindow *window)
 			  G_CALLBACK (galeon_window_tab_detached_cb), 
 			  window);
 
-	eel_gconf_notification_add(CONF_TABS_TABBED_EDGE,
-				   (GConfClientNotifyFunc)tabbed_position_gconf_changed_cb,
-				   notebook);
+	g_signal_connect (G_OBJECT (notebook), "tab_added", 
+			  G_CALLBACK (galeon_window_tab_added_cb), 
+			  window);
+
+	g_signal_connect (G_OBJECT (notebook), "tab_removed", 
+			  G_CALLBACK (galeon_window_tab_removed_cb), 
+			  window);
+ 
+	galeon_dnd_url_drag_dest_set_with_flags (GTK_WIDGET(notebook),
+						 GTK_DEST_DEFAULT_MOTION |
+						 GTK_DEST_DEFAULT_DROP);
+ 
+	g_signal_connect (G_OBJECT(notebook), "drag_data_received",
+			  G_CALLBACK(galeon_window_tab_drag_data_received_cb),
+			  NULL);
+ 
+	galeon_notification_add(CONF_TABS_TABBED_EDGE,
+				(GConfClientNotifyFunc)tabbed_position_gconf_changed_cb,
+				notebook, &window->priv->notifiers);
+
+	galeon_notification_add(CONF_TABS_TABBED_ALWAYS_SHOW,
+				(GConfClientNotifyFunc)tabbed_always_show_gconf_changed_cb,
+				notebook, &window->priv->notifiers);
     
 	gtk_widget_show (GTK_WIDGET (notebook));
 
@@ -501,6 +577,27 @@ setup_bookmarks_menu (GaleonWindow *window, GbBookmarkSet *bs)
 	return menu;
 }
 
+static GaleonCssMenu *
+setup_css_menu (GaleonWindow *window)
+{
+	GaleonWindowPrivate *p = window->priv;
+	GtkWidget *menu_item;
+	GtkWidget *menu_shell;
+	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
+	
+	menu_item = gtk_menu_item_new_with_mnemonic (_("St_yles"));
+	menu_shell = gtk_menu_new ();
+	gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item), menu_shell);
+	gtk_widget_show (menu_item);
+
+	gul_bonobo_add_numbered_widget (ui_component, menu_item, 0, CSS_MENU_PATH);
+
+	p->css_menu = galeon_css_menu_new (GTK_MENU_SHELL (menu_shell));
+	galeon_css_menu_set_parent_widget (p->css_menu, menu_item);
+
+	return p->css_menu;
+}
+
 static GbBonoboUIMenu *
 setup_bookmarks_context_menu (GaleonWindow *window, GbBookmarkSet *bs)
 {
@@ -605,7 +702,16 @@ view_fullscreen_changed_cb (BonoboUIComponent *component,
 {	
 	if (window->priv->ignore_layout_toggles) return;
 
-	galeon_window_toggle_fullscreen (window);
+	if (state == NULL || *state == '\0') return;
+
+	if (strcmp (state, "1") == 0)
+	{
+		gtk_window_fullscreen (GTK_WINDOW (window));
+	}
+	else
+	{
+		gtk_window_unfullscreen (GTK_WINDOW (window));
+	}
 }
 
 static void
@@ -626,6 +732,9 @@ update_layout_toggles(GaleonWindow *window)
 				    mask & EMBED_CHROME_STATUSBARON);
 	gul_bonobo_set_toggle_state(ui_component, VIEW_FULLSCREEN_PATH, 
 				    mask & EMBED_CHROME_OPENASFULLSCREEN);
+	toolbar_set_fullscreen (window->priv->toolbar,
+				mask & EMBED_CHROME_OPENASFULLSCREEN);
+
 
 	window->priv->ignore_layout_toggles = FALSE;
 }
@@ -771,10 +880,15 @@ accept_cookies_gconf_changed_cb(GConfClient *client,
 static void
 setup_web_menu (GaleonWindow *window)
 {
+	GConfValue *gcvalue;
+	
 	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT (window->ui_component);
 
-	radio_menu_group_set_from_value(ui_component, COOKIES,
-					eel_gconf_get_value(CONF_PERSISTENT_COOKIES_BEHAVIOR));
+	gcvalue = eel_gconf_get_value(CONF_PERSISTENT_COOKIES_BEHAVIOR);
+
+	radio_menu_group_set_from_value(ui_component, COOKIES, gcvalue);
+
+	gconf_value_free (gcvalue);
 
 	bonobo_ui_component_add_listener(ui_component, "AcceptCookiesAlways",
 					 accept_cookies_bonoboui_changed_cb,
@@ -786,9 +900,9 @@ setup_web_menu (GaleonWindow *window)
 					 accept_cookies_bonoboui_changed_cb,
 					 GINT_TO_POINTER(ACCEPT_COOKIES_NEVER));
 
-	eel_gconf_notification_add(CONF_PERSISTENT_COOKIES_BEHAVIOR, 
-				   (GConfClientNotifyFunc)accept_cookies_gconf_changed_cb, 
-				   ui_component);
+	galeon_notification_add(CONF_PERSISTENT_COOKIES_BEHAVIOR, 
+				(GConfClientNotifyFunc)accept_cookies_gconf_changed_cb, 
+				ui_component, &window->priv->notifiers);
 
 	gul_bonobo_set_toggle_state(ui_component, ALLOW_POPUPS_TOGGLE_PATH,
 				    eel_gconf_get_boolean (CONF_FILTERING_ALLOW_POPUPS));
@@ -810,18 +924,18 @@ setup_web_menu (GaleonWindow *window)
 					 (BonoboUIListenerFn)allow_setting_bonoboui_changed_cb,
 					 CONF_FILTERING_JAVA_ENABLED);
 
-	eel_gconf_notification_add(CONF_FILTERING_ALLOW_POPUPS,
-				   (GConfClientNotifyFunc)allow_popups_gconf_changed_cb,
-				   window);
-	eel_gconf_notification_add(CONF_FILTERING_ALLOW_POPUPS,
-				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
-				   ui_component);
-	eel_gconf_notification_add(CONF_FILTERING_JAVASCRIPT_ENABLED,
-				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
-				   ui_component);
-	eel_gconf_notification_add(CONF_FILTERING_JAVA_ENABLED,
-				   (GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
-				   ui_component);
+	galeon_notification_add(CONF_FILTERING_ALLOW_POPUPS,
+				(GConfClientNotifyFunc)allow_popups_gconf_changed_cb,
+				window, &window->priv->notifiers);
+	galeon_notification_add(CONF_FILTERING_ALLOW_POPUPS,
+				(GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
+				ui_component, &window->priv->notifiers);
+	galeon_notification_add(CONF_FILTERING_JAVASCRIPT_ENABLED,
+				(GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
+				ui_component, &window->priv->notifiers);
+	galeon_notification_add(CONF_FILTERING_JAVA_ENABLED,
+				(GConfClientNotifyFunc)allow_setting_gconf_changed_cb,
+				ui_component, &window->priv->notifiers);
 }
 
 static void
@@ -852,10 +966,13 @@ view_images_gconf_changed_cb(GConfClient *client,
 static void
 setup_images_menu(GaleonWindow *window)
 {
+	GConfValue *gcvalue;
 	BonoboUIComponent *ui_component = BONOBO_UI_COMPONENT(window->ui_component);
 
-	radio_menu_group_set_from_value(ui_component, IMAGES,
-					eel_gconf_get_value(CONF_FILTERING_IMAGE_LOADING_TYPE));
+	gcvalue = eel_gconf_get_value(CONF_FILTERING_IMAGE_LOADING_TYPE);
+	radio_menu_group_set_from_value(ui_component, IMAGES, gcvalue);
+
+	gconf_value_free (gcvalue);
 
 	bonobo_ui_component_add_listener(ui_component, "ViewImagesAlways",
 					 view_images_bonoboui_changed_cb,
@@ -867,9 +984,9 @@ setup_images_menu(GaleonWindow *window)
 					 view_images_bonoboui_changed_cb,
 					 GINT_TO_POINTER(VIEW_IMAGES_NEVER));
 
-	eel_gconf_notification_add(CONF_FILTERING_IMAGE_LOADING_TYPE, 
-				   (GConfClientNotifyFunc)view_images_gconf_changed_cb, 
-				   ui_component);
+	galeon_notification_add(CONF_FILTERING_IMAGE_LOADING_TYPE, 
+				(GConfClientNotifyFunc)view_images_gconf_changed_cb, 
+				ui_component, &window->priv->notifiers);
 }
 
 static void
@@ -958,6 +1075,9 @@ setup_sidebar (GaleonWindow *window)
 	gtk_paned_add2 (GTK_PANED(hpaned),
 			GTK_WIDGET(window->priv->notebook));
 
+	gul_state_monitor_paned (hpaned, STATE_SIDEBAR_SIZE,
+			         STATE_SIDEBAR_SIZE_DEFAULT);
+
 	window->priv->sidebar = sidebar;
 	
 	return hpaned;
@@ -995,7 +1115,8 @@ galeon_window_init (GaleonWindow *window)
 						     window);
 	setup_layout_menus (window);
 	setup_images_menu (window);
-	setup_web_menu(window);
+	setup_web_menu (window);
+	setup_css_menu (window);
 	
 	/* Setup the embed popups factory */
 	window->priv->embed_popup = setup_popup_factory (window, 
@@ -1093,6 +1214,8 @@ galeon_window_finalize (GObject *object)
 
         g_return_if_fail (window->priv != NULL);
 
+	galeon_notification_remove (&window->priv->notifiers);
+
 	remove_from_session (window);
 
 	if (window->priv->embed_popup)
@@ -1103,6 +1226,11 @@ galeon_window_finalize (GObject *object)
 	g_object_unref (G_OBJECT (window->priv->bookmarks_menu));
 	g_object_unref (G_OBJECT (window->priv->bookmarks_context_menu));
 	g_object_unref (G_OBJECT (window->priv->bookmarks_toolbar));
+	g_object_unref (G_OBJECT (window->priv->ppview_toolbar));
+	g_object_unref (G_OBJECT (window->priv->css_menu));
+
+	/* break circular references in the toolbar */
+	g_object_run_dispose (G_OBJECT(window->priv->toolbar));
 	g_object_unref (G_OBJECT (window->priv->toolbar));
 	g_object_unref (G_OBJECT (window->priv->statusbar));
 	
@@ -1129,6 +1257,8 @@ galeon_window_finalize (GObject *object)
 	g_object_unref (window->priv->recent_history);
 	g_object_unref (window->priv->recent_history_menu);
 
+	bonobo_object_unref (window->ui_component);
+
         g_free (window->priv);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -1162,57 +1292,6 @@ galeon_window_get_chrome (GaleonWindow *window)
 	return window->priv->chrome_mask;
 }
 
-static void
-wmspec_change_state (gboolean   add,
-                     GdkWindow *window,
-                     GdkAtom    state1,
-                     GdkAtom    state2)
-{
-	XEvent xev;
-
-	#define _NET_WM_STATE_REMOVE        0    /* remove/unset property */
-	#define _NET_WM_STATE_ADD           1    /* add/set property */
-	#define _NET_WM_STATE_TOGGLE        2    /* toggle property  */  
-  
-	xev.xclient.type = ClientMessage;
-  	xev.xclient.serial = 0;
-  	xev.xclient.send_event = True;
-  	xev.xclient.display = gdk_display;
-  	xev.xclient.window = GDK_WINDOW_XID (window);
-  	xev.xclient.message_type = gdk_x11_get_xatom_by_name ("_NET_WM_STATE");
-  	xev.xclient.format = 32;
-  	xev.xclient.data.l[0] = add ? _NET_WM_STATE_ADD : _NET_WM_STATE_REMOVE;
-  	xev.xclient.data.l[1] = gdk_x11_atom_to_xatom (state1);
-  	xev.xclient.data.l[2] = gdk_x11_atom_to_xatom (state2);
-  
-  	XSendEvent (gdk_display, GDK_WINDOW_XID (gdk_get_default_root_window ()),
-                    False,
-                    SubstructureRedirectMask | SubstructureNotifyMask,
-                    &xev);
-}
-
-static void
-window_set_fullscreen_mode (GaleonWindow *window, gboolean active)
-{
-        GdkWindow *gdk_window;
-
-        gdk_window = GTK_WIDGET(window)->window;
-
-        if (gdk_net_wm_supports (gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN",
-                                                  FALSE)))
-        {
-                wmspec_change_state (active,
-                                     gdk_window,
-                                     gdk_atom_intern ("_NET_WM_STATE_FULLSCREEN",
-                                                      FALSE),
-                                     GDK_NONE);
-        }
-        else
-        {
-                g_warning ("NET_WM_STATE_FULLSCREEN not supported");
-	}
-}
-
 static void
 translate_default_chrome (EmbedChromeMask *chrome_mask)
 {	
@@ -1268,6 +1347,34 @@ translate_default_chrome (EmbedChromeMask *chrome_mask)
 	}
 }
 
+/* Called after GaleonWindow has switched to fullscreen, setup toolbars etc. to
+ * match.
+ */
+static void
+galeon_window_fullscreen (GaleonWindow *window)
+{
+	EmbedChromeMask mask;
+
+	mask = window->priv->chrome_mask;
+	mask |= EMBED_CHROME_DEFAULT;
+	mask |= EMBED_CHROME_OPENASFULLSCREEN;
+	galeon_window_set_chrome (window, mask);
+}
+
+/* Called after GaleonWindow has switched back to normal size again, restore
+ * normal toolbar etc. layouts.
+ */
+static void
+galeon_window_unfullscreen (GaleonWindow *window)
+{
+	EmbedChromeMask mask;
+
+	mask = window->priv->chrome_mask;
+	mask |= EMBED_CHROME_DEFAULT;
+	mask &= ~EMBED_CHROME_OPENASFULLSCREEN;
+	galeon_window_set_chrome (window, mask);
+}
+
 void  
 galeon_window_set_chrome (GaleonWindow *window, 
 			  EmbedChromeMask flags)
@@ -1305,15 +1412,6 @@ galeon_window_set_chrome (GaleonWindow *window,
 	ppview_toolbar_set_visibility (window->priv->ppview_toolbar, 
 				       ppvtoolbar);
 	
-	/* set fullscreen only when it's really changed */
-	if ((window->priv->chrome_mask & EMBED_CHROME_OPENASFULLSCREEN) != 
-	    (flags & EMBED_CHROME_OPENASFULLSCREEN))
-	{
-		save_window_chrome (window);
-		window_set_fullscreen_mode (window,
-					    flags & EMBED_CHROME_OPENASFULLSCREEN);
-	}
-
 	gb_bonobo_ui_toolbar_set_visible (window->priv->bookmarks_toolbar,
 					  flags & EMBED_CHROME_PERSONALTOOLBARON);
 	
@@ -1327,12 +1425,14 @@ galeon_window_set_chrome (GaleonWindow *window,
 void
 galeon_window_toggle_fullscreen (GaleonWindow *window)
 {
-	EmbedChromeMask mask;
-	
-	mask = galeon_window_get_chrome (window);
-	mask ^= EMBED_CHROME_OPENASFULLSCREEN;
-	mask |= EMBED_CHROME_DEFAULT;
-	galeon_window_set_chrome (window, mask);
+	if (gdk_window_get_state (GTK_WIDGET (window)->window) & GDK_WINDOW_STATE_FULLSCREEN)
+	{
+		gtk_window_unfullscreen (GTK_WINDOW (window));
+	}
+	else
+	{
+		gtk_window_fullscreen (GTK_WINDOW (window));
+	}
 }
 
 void 
@@ -1419,8 +1519,6 @@ galeon_window_remove_tab (GaleonWindow *window,
 	g_return_if_fail (IS_GALEON_WINDOW (window));
 	g_return_if_fail (IS_GALEON_TAB (tab));
 
-	window->priv->active_tab = NULL;
-	
 	embed  = GTK_WIDGET (galeon_tab_get_embed (tab));
 	
 	gul_notebook_remove_page (GUL_NOTEBOOK (window->priv->notebook), 
@@ -1461,21 +1559,10 @@ galeon_window_load_url (GaleonWindow *window,
         g_return_if_fail (url != NULL);
 	
 	set = galeon_shell_get_bookmark_set (galeon_shell);
-	if (set)
-	{
-		real_url = gb_bookmark_set_get_url_by_nick_and_args (set, url);
-		if (!real_url)
-		{
-			/* no matching nick found */
-			real_url = g_strdup (url);
-		}
-	}
-	else
-	{
-		real_url = g_strdup (url);
-	}
+	real_url = gb_bookmark_set_get_url_by_nick_and_args (set, url);
+
+        galeon_embed_load_url (embed, real_url != NULL ? real_url : url);
 
-        galeon_embed_load_url (embed, real_url);
 	g_free (real_url);
 }
 
@@ -1488,7 +1575,6 @@ void
 galeon_window_show (GtkWidget *widget)
 {
 	GaleonWindow *window = GALEON_WINDOW(widget);
-	GaleonTab *tab;
 	int w = -1, h = -1;
 	
 	if (!window->priv->chrome_mask)
@@ -1496,14 +1582,20 @@ galeon_window_show (GtkWidget *widget)
 		galeon_window_set_chrome (window, EMBED_CHROME_DEFAULT);
 	}
 
-	tab = galeon_window_get_active_tab (window);
-	if (tab) galeon_tab_get_size (tab, &w, &h);
-	
-	if (!window->priv->has_default_size && w == -1 && h == -1) 
+	if (!window->priv->has_default_size &&
+	    window->priv->chrome_mask & EMBED_CHROME_OPENASPOPUP)
 	{
-		gul_state_load_window (GTK_WIDGET(window), 
-				       "main_window", 
-			               600, 500, TRUE);
+		GaleonTab *tab;
+
+		tab = galeon_window_get_active_tab (window);
+		if (tab) galeon_tab_get_size (tab, &w, &h);
+	}
+	
+	if (w == -1 && h == -1) 
+	{
+		gul_state_monitor_window (widget, "Main Window",
+				          STATE_MAIN_WIDTH_DEFAULT,
+					  STATE_MAIN_HEIGHT_DEFAULT);
 		window->priv->has_default_size = TRUE;
 	}
 	
@@ -1904,6 +1996,19 @@ update_permission_control (GaleonWindow *window)
 	}
 }
 
+static void
+update_css_menu_control (GaleonWindow *window)
+{
+	GaleonWindowPrivate *p;
+	GaleonEmbed *e;
+	g_return_if_fail (IS_GALEON_WINDOW (window));
+	p = window->priv;
+	g_return_if_fail (GALEON_IS_CSS_MENU (p->css_menu));
+
+	e = galeon_window_get_active_embed (window);
+	galeon_css_menu_set_embed (p->css_menu, e);
+}
+
 void  
 galeon_window_update_control (GaleonWindow *window,
 			      ControlID control)
@@ -1952,6 +2057,9 @@ galeon_window_update_control (GaleonWindow *window,
 	case PermissionControl:
 		update_permission_control(window);
 		break;
+	case CssMenuControl:
+		update_css_menu_control(window);
+		break;
 	default:
 		g_warning ("unknown control specified for updating");
 		break;
@@ -1976,6 +2084,7 @@ galeon_window_update_all_controls (GaleonWindow *window)
 		update_find_control (window);
 		update_spinner_control (window);
 		update_permission_control(window);
+		update_css_menu_control (window);
 	}
 }
 
@@ -2220,6 +2329,12 @@ galeon_window_bookmark_activate (GaleonWindow *w, GbBookmarkEventActivated *ev)
 		gdk_event_get_state (ev->event, &mod);
 		flags = galeon_shell_modifier_flags (mod);
 	}
+	else if (f == GB_BAF_EDITOR)
+	{
+		/* ignore it */
+		g_free (url);
+		return;
+	}
 	else
 	{
 		g_assert_not_reached ();
@@ -2448,6 +2563,10 @@ void
 galeon_window_site_visited (GaleonWindow *window, const gchar *url, const gchar *title)
 {
 	g_return_if_fail (IS_GALEON_WINDOW (window));
+
+	/* ignore about:blank, it's not shown in any other history either */
+	if (!url || !strcmp (url, "about:blank")) return;
+
 	galeon_window_recent_history_visited (window->priv->recent_history, url, title);
 }
 
@@ -2459,7 +2578,116 @@ galeon_window_recent_history_menu_activated_cb (GaleonWindowRecentHistoryMenu *w
 	galeon_window_load_url (w, url);
 }
 
+
+/**
+ *  Clear the recent history menu
+ */
 void
+galeon_window_clear_history (GaleonWindow *window)
+{
+	GList *tabs, *item;
+
+	g_return_if_fail (IS_GALEON_WINDOW (window));
+
+	galeon_window_recent_history_clear (window->priv->recent_history);
+
+	tabs = galeon_window_get_tabs (window);
+	for (item = tabs ; item ; item = item->next)
+	{
+		GaleonTab *tab = GALEON_TAB (item->data);
+		GaleonEmbed *embed = galeon_tab_get_embed (tab);
+
+		galeon_embed_shistory_clear (embed);
+	}
+	g_list_free (tabs);
+
+	update_nav_control (window);
+}
+
+
+/**
+ *  Little helper structure that we use while iterating through the
+ *  dropped urls
+ */
+typedef struct {
+	GaleonEmbed * embed;
+	GaleonWindow * window;
+	GaleonTab * tab;
+} UrlReceiveContext;
+
+
+static void
+each_url_receive_data_binder (const char * url, const char * title, 
+			      gpointer data )
+{
+	UrlReceiveContext * context = (UrlReceiveContext*)data;
+	GaleonEmbed * embed = context->embed;
+
+	if (embed != NULL)
+	{
+		/**
+		 * The first url is special: if the drag was to an
+		 * existing tab, load it there
+		 */
+		galeon_embed_load_url (embed, url);
+		context->embed = NULL;
+	}
+	else
+	{
+		context->tab =
+			galeon_shell_new_tab (galeon_shell, context->window,
+					      context->tab, url,
+					      GALEON_NEW_TAB_IN_EXISTING_WINDOW |
+					      GALEON_NEW_TAB_APPEND);
+	}
+}
+
+
+static void
+galeon_window_tab_drag_data_received_cb(GtkWidget *widget,
+					GdkDragContext *dc,
+					gint x, gint y,
+					GtkSelectionData *selection_data,
+					guint info,
+					guint t,
+					GtkWidget * child)
+{
+	UrlReceiveContext context;
+	GtkWidget * toplevel;
+	context.embed = child ? GALEON_EMBED (child) : 0;
+	context.tab = 0;
+
+	toplevel = gtk_widget_get_toplevel (widget);
+	g_return_if_fail (IS_GALEON_WINDOW (toplevel));
+	
+	context.window = GALEON_WINDOW(toplevel);
+
+	galeon_dnd_drag_data_receive (widget, dc, x, y, selection_data,
+			 info, t, &context, each_url_receive_data_binder);
+}
+
+
+static void		
+galeon_window_tab_added_cb  (GulNotebook *notebook, GtkWidget *child, GaleonWindow *window)
+{
+	GtkWidget * label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (notebook), child);
+
+	galeon_dnd_url_drag_dest_set (label);
+
+	g_signal_connect (G_OBJECT(label), "drag_data_received",
+			  G_CALLBACK(galeon_window_tab_drag_data_received_cb),
+			  child);
+}
+
+
+static void		
+galeon_window_tab_removed_cb (GulNotebook *notebook, GtkWidget *child, GaleonWindow *window)
+{
+	update_spinner_control (window);
+}
+
+
+static void
 galeon_window_tab_detached_cb (GulNotebook *notebook, gint page, 
 			        gint x, gint y, GaleonWindow *window)
 {
@@ -2493,11 +2721,26 @@ galeon_window_set_tab_status (GaleonWindow	*window,
 			      TabLoadStatus	 status)
 {
 	GdkColor *color;
+	GtkWidget *icon = galeon_tab_get_icon(tab);
+	gboolean have_ref = FALSE;
 
 	switch (status)
 	{
 	case TAB_LOAD_STARTED:
 		color = &_galeon_window_loading_tab_color;
+
+		/* Don't display the loading animation if favicons in tabs are
+		 * disabled. If they are disabled, then the icon retreived above
+		 * will be empty, so won't get displayed */
+		if (eel_gconf_get_integer (CONF_TABS_FAVICON))
+		{
+			icon = gtk_image_new_from_animation (_galeon_window_loading_animation);
+			gtk_widget_show (icon);
+			g_object_ref (G_OBJECT (icon));
+			gtk_object_sink (GTK_OBJECT (icon));
+			have_ref = TRUE;
+		}
+
         	break;
 	case TAB_LOAD_COMPLETED:
 		color = &_galeon_window_new_tab_color;
@@ -2508,9 +2751,18 @@ galeon_window_set_tab_status (GaleonWindow	*window,
 		break;
 	}
 
+	gul_notebook_set_page_icon( GUL_NOTEBOOK (window->priv->notebook),
+				    GTK_WIDGET (galeon_tab_get_embed (tab)),
+				    icon );
+
 	gul_notebook_set_page_color (GUL_NOTEBOOK (window->priv->notebook),
 			             GTK_WIDGET (galeon_tab_get_embed (tab)),
 				     color);
+	
+	if (have_ref)
+	{
+		g_object_unref (G_OBJECT(icon));
+	}
 }
 
 void
