@@ -238,5 +238,11 @@ void popup_cmd_view_source (BonoboUIComponent *uic,
 			    GaleonEmbedPopup *popup, 
 			    const char* verbname)
 {
-	/* FIXME implement */
+	GaleonWindow *window;
+	GaleonEmbed * embed = galeon_embed_popup_get_embed (popup);
+	window = get_window_from_popup (popup);
+
+	galeon_shell_new_tab_from_embed
+		(galeon_shell, window, embed, NULL,
+		 GALEON_NEW_TAB_VIEW_SOURCE);
 }
