@@ -22,11 +22,23 @@
  * code in there if possible, so other apps like gedit or profterm can use 
  * it easily if they want.
  */
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "gul-notebook.h"
+#include "gul-gui.h"
 #include "galeon-marshal.h"
+#include "eel-input-event-box.h"
+
+#include <gtk/gtklabel.h>
+#include <gtk/gtkhbox.h>
+#include <gtk/gtkbutton.h>
+#include <gtk/gtktooltips.h>
+#include <gtk/gtkstock.h>
+#include <gtk/gtkimage.h>
+#include <gtk/gtkmain.h>
 
-#include <gtk/gtk.h>
 #include <glib-object.h>
 #include <libgnome/gnome-i18n.h>
 
@@ -40,6 +52,8 @@ struct GulNotebookPrivate
 	GList *focused_pages;
 	GList *opened_tabs;
 	GtkPolicyType policy;
+	GtkTooltips *title_tips;
+	gboolean automatic_tab_switch;
 	
 	/* Used during tab drag'n'drop */
 	gulong motion_notify_handler_id;
@@ -72,7 +86,9 @@ static gboolean motion_notify_cb (GulNotebook *notebook,
 /* Signals */
 enum
 {
-	TAB_DROPPED,
+	TAB_ADDED,
+	TAB_REMOVED,
+	TABS_REORDERED,
 	TAB_DETACHED,
 	LAST_SIGNAL
 };
@@ -114,19 +130,26 @@ gul_notebook_class_init (GulNotebookClass *klass)
 	object_class->finalize = gul_notebook_finalize;
 
 	/* init signals */
-	gul_notebook_signals[TAB_DROPPED] =
-		g_signal_new ("tab_dropped",
+	gul_notebook_signals[TAB_ADDED] =
+		g_signal_new ("tab_added",
 			      G_OBJECT_CLASS_TYPE (object_class),
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GulNotebookClass, 
-					       tab_dropped),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GulNotebookClass, tab_added),
 			      NULL, NULL,
-			      galeon_marshal_VOID__OBJECT_OBJECT_INT,
+			      galeon_marshal_VOID__OBJECT,
 			      G_TYPE_NONE,
-			      3, 
-			      GTK_TYPE_WIDGET,
-			      GUL_NOTEBOOK_TYPE,
-			      G_TYPE_INT);
+			      1,
+			      GTK_TYPE_WIDGET);
+	gul_notebook_signals[TAB_REMOVED] =
+		g_signal_new ("tab_removed",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GulNotebookClass, tab_removed),
+			      NULL, NULL,
+			      galeon_marshal_VOID__OBJECT,
+			      G_TYPE_NONE,
+			      1,
+			      GTK_TYPE_WIDGET);
 	gul_notebook_signals[TAB_DETACHED] =
 		g_signal_new ("tab_detached",
 			      G_OBJECT_CLASS_TYPE (object_class),
@@ -140,6 +163,16 @@ gul_notebook_class_init (GulNotebookClass *klass)
 			      G_TYPE_INT,
 			      G_TYPE_INT,
 			      G_TYPE_INT);
+	gul_notebook_signals[TABS_REORDERED] =
+		g_signal_new ("tabs_reordered",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (GulNotebookClass, tabs_reordered),
+			      NULL, NULL,
+			      galeon_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
 
 }
 
@@ -303,6 +336,19 @@ tab_get_label (GulNotebook *nb, GtkWidget *child)
 	return label;
 }
 
+
+static GtkWidget *
+tab_get_icon (GulNotebook *nb, GtkWidget *child)
+{
+	GtkWidget *hbox, *icon;
+
+	hbox = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb),
+					   child);
+	icon = g_object_get_data (G_OBJECT (hbox), "icon");
+
+	return icon;
+}
+
 static void
 tab_label_size_request_cb (GtkWidget *window,
 			   GtkRequisition *requisition,
@@ -323,24 +369,42 @@ void
 gul_notebook_move_page (GulNotebook *src, GulNotebook *dest, 
 			GtkWidget *src_page,  gint dest_page)
 {
-	GtkWidget *tab_label;
-	GtkWidget *menu_label;
+	GtkWidget *tab_label, *icon;
+	const gchar * text;
 
-	tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (src), src_page);
-	menu_label = gtk_notebook_get_menu_label (GTK_NOTEBOOK (src), src_page);
+	if (dest == NULL || src == dest)
+	{
+		gtk_notebook_reorder_child (GTK_NOTEBOOK (src), src_page, dest_page);
+		
+		if (src->priv->drag_in_progress == FALSE)
+		{
+			g_signal_emit (G_OBJECT (src), 
+				       gul_notebook_signals[TABS_REORDERED], 0);
+		}
+		return;
+	}
+
+	tab_label = tab_get_label (src, src_page);
+	icon      = tab_get_icon  (src, src_page);
 
 	/* We don't want gtk to destroy tab and src_page behind our back */
 	g_object_ref (G_OBJECT (src_page));
 	g_object_ref (G_OBJECT (tab_label));
-	g_object_ref (G_OBJECT (menu_label));
+	if (icon) g_object_ref (G_OBJECT (icon));
+
 	gul_notebook_remove_page (GUL_NOTEBOOK (src), src_page);
 	gul_notebook_insert_page (GUL_NOTEBOOK (dest), src_page,
 				  dest_page, TRUE);
-	gtk_notebook_set_tab_label (GTK_NOTEBOOK (dest), src_page, tab_label);
-	gtk_notebook_set_menu_label (GTK_NOTEBOOK (dest), src_page, menu_label);
+	/* Restore the text  */
+	text = gtk_label_get_text( GTK_LABEL(tab_label) );
+	gul_notebook_set_page_title( dest, src_page, text );
+	if (icon)
+	{
+		gul_notebook_set_page_icon (dest, src_page, icon);
+		g_object_unref (G_OBJECT (icon));
+	}
 	g_object_unref (G_OBJECT (src_page));
 	g_object_unref (G_OBJECT (tab_label));
-	g_object_unref (G_OBJECT (menu_label));
 }
 
 
@@ -460,16 +524,6 @@ button_release_cb (GulNotebook *notebook, GdkEventButton *event,
 				       cur_page_num, (gint)event->x_root, 
 				       (gint)event->y_root);
 		}
-		else
-		{
-			/* Tab was dragged and dropped (but it may have stayed
-			   in the same place) */
-			g_signal_emit (G_OBJECT(notebook), 
-				       gul_notebook_signals[TAB_DROPPED], 0,
-				       cur_page, 
-				       notebook->priv->src_notebook, 
-				       notebook->priv->src_page);
-		}
 
 		/* ungrab the pointer if it's grabbed */
 		if (gdk_pointer_is_grabbed ())
@@ -577,12 +631,20 @@ gul_notebook_switch_page_cb (GtkNotebook *notebook,
 	
 	child = gtk_notebook_get_nth_page (notebook, page_num);
 
-	/* Remove the old page, we dont want to grow unnecessarily
-	 * the list */
-	if (nb->priv->focused_pages)
+	if (nb->priv->automatic_tab_switch)
 	{
-		nb->priv->focused_pages = 
-			g_list_remove (nb->priv->focused_pages, child);
+		/* Remove the old page, we dont want to grow unnecessarily
+		 * the list */
+		if (nb->priv->focused_pages)
+		{
+			nb->priv->focused_pages = 
+				g_list_remove (nb->priv->focused_pages, child);
+		}
+	}
+	else
+	{
+		g_list_free (nb->priv->focused_pages);
+		nb->priv->focused_pages = NULL;
 	}
 
 	nb->priv->focused_pages = g_list_append (nb->priv->focused_pages,
@@ -598,11 +660,16 @@ gul_notebook_init (GulNotebook *notebook)
 {
 	notebook->priv = g_new (GulNotebookPrivate, 1);
 
+	notebook->priv->title_tips = gtk_tooltips_new ();
+	g_object_ref (G_OBJECT (notebook->priv->title_tips));
+	gtk_object_sink (GTK_OBJECT (notebook->priv->title_tips));
+
 	notebook->priv->drag_in_progress = FALSE;
 	notebook->priv->motion_notify_handler_id = 0;
 	notebook->priv->src_notebook = NULL;
 	notebook->priv->src_page = -1;
 	notebook->priv->focused_pages = NULL;
+	notebook->priv->automatic_tab_switch = FALSE;
 	notebook->priv->opened_tabs = NULL;
 	notebook->priv->policy = GTK_POLICY_AUTOMATIC;
 	
@@ -626,12 +693,10 @@ gul_notebook_finalize (GObject *object)
 
 	notebooks = g_list_remove (notebooks, notebook);
 
-	if (notebook->priv->focused_pages)
-	{
-		g_list_free (notebook->priv->focused_pages);
-	}
+	g_list_free (notebook->priv->focused_pages);
 	g_list_free (notebook->priv->opened_tabs);
-	
+	g_object_unref (notebook->priv->title_tips);
+
 	g_free (notebook->priv);
 }
 
@@ -658,34 +723,45 @@ tab_close_button_state_changed_cb (GtkWidget	*widget,
 static GtkWidget *
 tab_build_label (GulNotebook *nb, GtkWidget *child)
 {
-	GtkWidget *label, *hbox, *close_button, *image;
-	int h, w;
+	GtkWidget *label, *hbox, *close_button, *image, *label_hbox;
+	GtkRequisition size;
+	GtkRcStyle *rcstyle;
 	GClosure *closure;
 	GtkWidget *window;
+	GtkWidget *label_ebox;
 
 	window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
 	
-	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &w, &h);
-	
-	/* set hbox spacing and label padding (see below) so that there's an
-	 * equal amount of space around the label */
-	hbox = gtk_hbox_new (FALSE, 4);
+	/* Add some space between the label and the close button, this
+	 * should be the same as the spacing in the second hbox, and the
+	 * padding in the gtk_box_pack_end */
+	hbox = gtk_hbox_new (FALSE, 3);
 
-	/* setup close button */
+	/* setup close button, zero out {x,y}thickness to get smallest possible
+	 * size */
 	close_button = gtk_button_new ();
 	gtk_button_set_relief (GTK_BUTTON (close_button), 
 			       GTK_RELIEF_NONE);
+	rcstyle = gtk_rc_style_new ();
+	rcstyle->xthickness = rcstyle->ythickness = 0;
+	gtk_widget_modify_style (close_button, rcstyle);
+	gtk_rc_style_unref (rcstyle),
+
 	image = gtk_image_new_from_stock (GTK_STOCK_CLOSE,
 					  GTK_ICON_SIZE_MENU);
-	gtk_widget_set_size_request (close_button, w+2, h+2);
-	gtk_container_add (GTK_CONTAINER (close_button), 
-			   image);
-		
+	gtk_widget_size_request (image, &size);
+	gtk_widget_set_size_request (close_button, size.width, size.height);
+	gtk_container_add (GTK_CONTAINER (close_button), image);
+
 	/* setup label */ 
+	label_ebox = eel_input_event_box_new ();
+	label_hbox = gtk_hbox_new (FALSE, 3);
         label = gtk_label_new (_("Untitled"));
 	gtk_misc_set_alignment (GTK_MISC (label), 0.00, 0.5);
-        gtk_misc_set_padding (GTK_MISC (label), 4, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+	gtk_box_pack_end (GTK_BOX (label_hbox), label, TRUE, TRUE, 3);
+
+	gtk_box_pack_start (GTK_BOX (hbox), label_ebox, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (label_ebox), label_hbox);
 
 	tab_label_set_size (GTK_WIDGET (window), hbox);
 		
@@ -714,11 +790,16 @@ tab_build_label (GulNotebook *nb, GtkWidget *child)
                           child);
 		
 	gtk_widget_show (hbox);
+	gtk_widget_show (label_ebox);
+	gtk_widget_show (label_hbox);
 	gtk_widget_show (label);
 	gtk_widget_show (image);
 	gtk_widget_show (close_button);
 
 	g_object_set_data (G_OBJECT (hbox), "label", label);
+	g_object_set_data (G_OBJECT (hbox), "label-ebox", label_ebox);
+	g_object_set_data (G_OBJECT (hbox), "label-hbox", label_hbox);
+	g_object_set_data (G_OBJECT (hbox), "icon", 0);
 	
 	return hbox;
 }
@@ -788,23 +869,23 @@ gul_notebook_insert_page (GulNotebook *nb,
 
 	if (jump_to)
 	{
+		nb->priv->automatic_tab_switch = TRUE;
+
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (nb),
 					       position);
-		g_object_set_data (G_OBJECT (child), "jump_to", 
-				   GINT_TO_POINTER (jump_to));
+
+		nb->priv->automatic_tab_switch = FALSE;
 	}
+
+	g_signal_emit (G_OBJECT (nb), gul_notebook_signals[TAB_ADDED], 
+		       0, child);
 }
 
 static void
 smart_tab_switching_on_closure (GulNotebook *nb,
 			  	GtkWidget *child)
 {
-	gboolean jump_to;
-	
-	jump_to = GPOINTER_TO_INT (g_object_get_data 
-				   (G_OBJECT (child), "jump_to"));
-
-	if (!jump_to || !nb->priv->focused_pages)
+	if (!nb->priv->focused_pages)
 	{
 		gtk_notebook_next_page (GTK_NOTEBOOK (nb));
 	}
@@ -819,8 +900,13 @@ smart_tab_switching_on_closure (GulNotebook *nb,
 		child = GTK_WIDGET (l->data);
 		page_num = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
 						  child);
+
+		nb->priv->automatic_tab_switch = TRUE;
+		
 		gtk_notebook_set_current_page 
 			(GTK_NOTEBOOK (nb), page_num);
+
+		nb->priv->automatic_tab_switch = FALSE;
 	}
 }
 
@@ -830,12 +916,14 @@ gul_notebook_remove_page (GulNotebook *nb,
 {
 	int position, cur;
 	gboolean last_tab;
+	GtkWidget *label, *ebox;
 
-	last_tab = gtk_notebook_get_nth_page (GTK_NOTEBOOK (nb), 1) == NULL;
+	last_tab = gtk_notebook_get_n_pages (GTK_NOTEBOOK (nb)) == 1;
 	if (last_tab)
 	{
 		GtkWidget *window;
 		window = gtk_widget_get_toplevel (GTK_WIDGET (nb));
+		gul_gui_widget_hide_now (window);
 		gtk_widget_destroy (window);
 		return;
 	}
@@ -855,9 +943,30 @@ gul_notebook_remove_page (GulNotebook *nb,
 		smart_tab_switching_on_closure (nb, child);
 	}
 	
+ 	/* remove the tool tip for this ebox */
+ 	label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
+ 	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-ebox"));
+ 	gtk_tooltips_set_tip (GTK_TOOLTIPS (nb->priv->title_tips), ebox, NULL, NULL);
+ 
+	/* If we don't clear the icon then for some reason it gets reset back
+	 * to an empty icon. This means that although gul_notebook_move_page() has
+	 * a ref to the icon, when it is placed in the new notebook, it is empty */
+	gul_notebook_set_page_icon (nb, child, 0);
+
+	/**
+	 * we ref the child so that it's still alive while the tabs_removed
+	 * signal is processed.
+	 */
+	g_object_ref (child);
+ 
 	gtk_notebook_remove_page (GTK_NOTEBOOK (nb), position);
 
 	update_tabs_visibility (nb, FALSE);
+ 
+	g_signal_emit (G_OBJECT (nb), gul_notebook_signals[TAB_REMOVED], 
+		       0, child);
+ 
+	g_object_unref (child);
 }
 
 void
@@ -881,11 +990,44 @@ gul_notebook_set_page_title (GulNotebook *nb,
 			     GtkWidget *child,
 			     const char *title)
 {
-	GtkWidget *label;
-	
-	label = tab_get_label (nb, child);
+	GtkWidget *label, *ebox, *tab_label;
+
+	tab_label = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
+
+	ebox = GTK_WIDGET (g_object_get_data (G_OBJECT (tab_label), "label-ebox"));
+	label = GTK_WIDGET (g_object_get_data (G_OBJECT (tab_label), "label"));
+
 	gtk_label_set_label (GTK_LABEL (label), title);
 	gtk_notebook_set_menu_label_text (GTK_NOTEBOOK (nb), child, title);
+	gtk_tooltips_set_tip (nb->priv->title_tips, ebox, title, NULL);
+}
+
+
+void
+gul_notebook_set_page_icon (GulNotebook *nb,
+			    GtkWidget *child,
+			    GtkWidget *icon)
+{
+	GtkWidget *old_icon;
+	GtkWidget *label, *hbox;
+
+	label    = gtk_notebook_get_tab_label (GTK_NOTEBOOK (nb), child);
+	hbox     = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "label-hbox"));
+	old_icon = GTK_WIDGET (g_object_get_data (G_OBJECT (label), "icon"));
+
+	if (icon == old_icon) return;
+
+	if (old_icon)
+	{
+		gtk_container_remove (GTK_CONTAINER (hbox), old_icon);
+	}
+
+	if (icon)
+	{
+		gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+	}
+
+	g_object_set_data (G_OBJECT (label), "icon", icon);
 }
 
 void
